@@ -1,10 +1,12 @@
-"""Parser for Qingping BLE advertisements.
+"""
+Parser for Qingping BLE advertisements.
 
 This file is shamelessly copied from the following repository:
 https://github.com/Ernst79/bleparser/blob/c42ae922e1abed2720c7fac993777e1bd59c0c93/package/bleparser/qingping.py
 
 MIT License applies.
 """
+
 from __future__ import annotations
 
 import logging
@@ -21,7 +23,6 @@ _LOGGER = logging.getLogger(__name__)
 
 @dataclass
 class QingpingDevice:
-
     model: str
     name: str
 
@@ -31,13 +32,17 @@ DEVICE_TYPES = {
     0x04: QingpingDevice("CGH1", "Door/Window Sensor"),  # Door/Window Sensor
     0x07: QingpingDevice("CGG1", ""),
     0x09: QingpingDevice("CGP1W", ""),
-    0x16: QingpingDevice("CGG1", "Qingping Temp RH M"),
-    0x12: QingpingDevice("CGPR1", "Motion & Light"),
-    0x1E: QingpingDevice("CGC1", "BT Clock Lite"),
     0x0C: QingpingDevice("CGD1", "Alarm Clock"),
     0x0E: QingpingDevice("CGDN1", "Air Monitor Lite"),
-    0x24: QingpingDevice("CGDN1", "Air Monitor Lite"),
     0x0F: QingpingDevice("CGM1", "Lee Guitars Thermo-Hygrometer"),
+    0x12: QingpingDevice("CGPR1", "Motion & Light"),
+    0x15: QingpingDevice("CGF1W", "Temp RH Pro E"),
+    0x16: QingpingDevice("CGG1", "Temp RH M"),
+    0x18: QingpingDevice("CGP23W", "Temp & RH Monitor Pro"),
+    0x1E: QingpingDevice("CGC1", "BT Clock Lite"),
+    0x24: QingpingDevice("CGDN1", "Air Monitor Lite"),
+    0x33: QingpingDevice("CGP22C", "CO2 Temp RH"),
+    0x5D: QingpingDevice("CGP22C", "CO2 Temp RH"),
 }
 
 
@@ -85,7 +90,7 @@ class QingpingBluetoothDeviceData(BluetoothData):
                 self._process_xdata(
                     xdata_id,
                     xdata_size,
-                    data[xdata_point : xdata_point + xdata_size],  # noqa: E203
+                    data[xdata_point : xdata_point + xdata_size],
                 )
             xdata_point += xdata_size + 2
 
@@ -97,6 +102,17 @@ class QingpingBluetoothDeviceData(BluetoothData):
         elif xdata_id == 0x02 and xdata_size == 1:
             batt = unpack("B", xdata)[0]
             self.update_predefined_sensor(SensorLibrary.BATTERY__PERCENTAGE, batt)
+        elif xdata_id == 0x04 and xdata_size == 1:
+            closed = unpack("B", xdata)[0]
+            self.update_predefined_binary_sensor(
+                BinarySensorDeviceClass.DOOR, closed in (0, 2)
+            )
+            self.update_predefined_binary_sensor(
+                BinarySensorDeviceClass.PROBLEM,
+                closed == 2,  # left open
+                key=(None, "door_left_open"),
+                name="Door left open",
+            )
         elif xdata_id == 0x07 and xdata_size == 2:
             pressure = unpack("<H", xdata)[0]
             self.update_predefined_sensor(SensorLibrary.PRESSURE__MBAR, pressure / 10)
@@ -108,11 +124,6 @@ class QingpingBluetoothDeviceData(BluetoothData):
             self.update_predefined_sensor(
                 SensorLibrary.LIGHT__LIGHT_LUX, illuminance_1 + illuminance_2
             )
-        elif xdata_id == 0x04 and xdata_size == 1:
-            closed = unpack("B", xdata)[0]
-            self.update_predefined_binary_sensor(
-                BinarySensorDeviceClass.DOOR, not bool(closed)
-            )
         elif xdata_id == 0x09 and xdata_size == 4:
             illuminance = unpack("<I", xdata)[0]
             self.update_predefined_sensor(SensorLibrary.LIGHT__LIGHT_LUX, illuminance)
@@ -130,13 +141,13 @@ class QingpingBluetoothDeviceData(BluetoothData):
                 SensorLibrary.PM10__CONCENTRATION_MICROGRAMS_PER_CUBIC_METER, pm10
             )
         elif xdata_id == 0x13 and xdata_size == 2:
-            co2 = unpack("<H", xdata)[0]  # noqa: F841
+            co2 = unpack("<H", xdata)[0]
             self.update_predefined_sensor(
                 SensorLibrary.CO2__CONCENTRATION_PARTS_PER_MILLION, co2
             )
         elif xdata_id == 0x0F and xdata_size == 1:
             pass
-            # packet_id = unpack("B", xdata)[0]  # noqa: F841
+            # packet_id = unpack("B", xdata)[0]
             # result.update({"packet": packet_id})
         else:
             _LOGGER.debug(
